#!BPY

""" Registration info for Blender menus:
Name: 'SMD for M&B (.smd) - horse ...'
Blender: 242
Group: 'Export'
Submenu: 'Selected as reference model...' sel
Tip: 'Export to BRF-readable (.smd) format, horse skeleton.'
"""

__author__ = "Pelle Johnsen, edited by Eric Finley (Hellequin)"
__version__ = "Original 2.36 04/17/05, Edited 2.42 02/12/07"

__bpydoc__ = """\
This script exports Blender meshes to HalfLife2's .smd file format.

Supported:<br>

Missing:<br>
    Animation

Known issues:<br>
    Only interfaces with BRFEdit 0.8.2 and M&B 0.8xx horse skeleton.
	Doesn't actually read the skeleton data, so if you've changed it, you'll still
	bind to the bones in their original locations.

Notes:<br>
    
"""

# $Id$
#
# --------------------------------------------------------------------------
# smd_export version 2.36.1
# Program versions: Blender 2.36 and HalfLife2
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2006: Pelle Johnsen, pelle.johnsen _at_ mail.dk
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

import Blender, meshtools, math, time
from Blender import Armature, Object, Scene, Mathutils, NMesh, Material, Image, Window, Modifier
from Blender.Mathutils import *

def writeTri(file, object, face, verts):
	file.write("Textures/%s\n" % face.image.name)
	for vi in verts:
		v = face.v[vi]
		vec = Vector([v.co[0], v.co[1], v.co[2], 1])
		vec = vec * object.getMatrix('worldspace')
		
		weights = object.getData().getVertexInfluences(v.index)
		maxWeight = 0.0
		parentBoneName = "0"
		if len(weights) > 0:
			for wi in weights:
				if wi[1] > maxWeight:
					maxWeight = wi[1]
					parentBoneName = wi[0]				

		file.write("%s %f %f %f " % (parentBoneName, vec.x, vec.y, vec.z))
		if face.smooth:
			vec = Vector([v.no[0], v.no[1], v.no[2], 1])
			vec = vec * object.getMatrix('worldspace')
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		else:
			vec = Vector([face.no[0], face.no[1], face.no[2], 1])
			vec = vec * object.getMatrix('worldspace')
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		file.write("%f %f " % (face.uv[vi][0], face.uv[vi][1]))
		
		if len(weights) > 0:
			file.write("%d " % len(weights))
			for wi in weights:
				file.write("%s %f " % (wi[0], wi[1]))
		file.write("\n")

class SMDExport:

	def __init__(self, filename):
		print 'Trying SMDExport...'

		objects = Blender.Object.GetSelected()
	
		try:
			file = open(filename, 'w')
		except IOError, (errno, strerror):
			errmsg = "IOError #%s" % errno
			errmsg = errmsg + "%t|" + strerror
			Blender.Draw.PupMenu(errmsg)
			return None
		file.write('version 1\n')
		file.write('nodes\n')
		file.write('0 "0" -1\n')
		file.write('1 "1" 0\n')
		file.write('2 "2" 1\n')
		file.write('3 "3" 2\n')
		file.write('4 "4" 3\n')
		file.write('5 "9" 1\n')
		file.write('6 "10" 5\n')
		file.write('7 "11" 6\n')
		file.write('8 "12" 7\n')
		file.write('9 "5" 1\n')
		file.write('10 "6"  9\n')
		file.write('11 "7" 10\n')
		file.write('12 "8" 11\n')
		file.write('13 "13"  0\n')
		file.write('14 "14" 13\n')
		file.write('15 "15" 14\n')
		file.write('16 "16" 15\n')
		file.write('17 "17"  0\n')
		file.write('18 "18" 17\n')
		file.write('19 "19" 18\n')
		file.write('20 "20" 19\n')
		file.write('21 "21"  0\n')
		file.write('22 "22" 21\n')
		file.write('end\n')
		file.write('skeleton\n')
		file.write('time 0\n')
		file.write('0 0.000000 -5.582998 14.382000 1.276920 0.301966 1.598354\n')
		file.write('1 5.775710 1.630216 0.667203 -0.194376 -0.023825 0.007994\n')
		file.write('2 5.351058 1.992543 1.114654 0.222517 -0.447221 1.076280\n')
		file.write('3 3.174880 -0.000001 0.000000 0.037465 -0.000000 -0.081695\n')
		file.write('4 2.568381 0.000001 0.000001 1.543624 -0.025794 -0.759186\n')
		file.write('5 6.260594 -0.287045 -1.867620 0.184965 -0.484326 1.324361\n')
		file.write('6 -3.236868 -0.000083 -0.001114 -0.067426 0.031473 0.440849\n')
		file.write('7 -4.293035 0.000002 0.000000 -0.020493 -0.000000 -0.067115\n')
		file.write('8 -2.816358 0.000001 -0.000001 1.568116 -3.121246 -1.440365\n')
		file.write('9 6.242899 -1.705240 0.752505 0.184965 -0.484326 1.324361\n')
		file.write('10 -3.236869 -0.000083 -0.001115 -0.067427 0.031473 0.440849\n')
		file.write('11 -4.293037 -0.000002 0.000001 -0.020492 0.000000 -0.067114\n')
		file.write('12 -2.816361 0.000000 -0.000000 1.568130 -3.121253 -1.440590\n')
		file.write('13 -0.834039 -0.372079 -1.755347 0.174455 0.199838 -0.792473\n')
		file.write('14 4.231141 -0.000001 -0.000000 0.056609 0.041012 -0.839230\n')
		file.write('15 5.267721 -0.000001 -0.000001 -0.002390 0.008635 0.431503\n')
		file.write('16 3.356640 0.000019 0.001159 -1.569692 0.015573 1.522049\n')
		file.write('17 -0.912419 -1.258138 1.088137 0.174455 0.199838 -0.792472\n')
		file.write('18 4.231135 -0.000003 -0.000000 0.056609 0.041012 -0.839231\n')
		file.write('19 5.267724 -0.000001 -0.000001 -0.002390 0.008289 0.431508\n')
		file.write('20 3.356638 0.000000 -0.000000 -1.570023 0.015591 1.521919\n')
		file.write('21 -1.693311 0.292660 0.044521 2.296783 0.025898 -0.171133\n')
		file.write('22 -0.948596 0.864667 1.916208 -0.196843 0.980348 -0.739549\n')
		file.write('end\n')
		file.write('triangles\n')
		
		for object in objects:
			if object.getType() != 'Mesh': continue
			print 'object:' + object.name
			mesh = object.getData()
			if not mesh.hasFaceUV():
				Blender.Draw.PupMenu("ERROR!%t|Meshses must have face UVs.|See Blender guide under UV editor.")
				return None
			faces = mesh.faces
			for face in faces:
				writeTri(file, object, face, [0, 1, 2])
				if len(face.v) == 4:
					writeTri(file, object, face, [0, 2, 3])

		file.write("end\n")
		file.close()

		print "Done. Saved to %s\n" % filename
			

# -- end of class SMDExport

def fs_callback(filename):
	if filename.find('.smd', -4) <= 0: filename += '.smd'
	test = SMDExport(filename)

if __script__['arg'] == 'config':
	exit
else:
	fname = Blender.sys.splitext(Blender.Get('filename'))[0] + '.smd'
	Blender.Window.FileSelector(fs_callback, "Export SMD", fname)