#!BPY

""" Registration info for Blender menus:
Name: 'SMD for M&B (.smd) - human ...'
Blender: 242
Group: 'Export'
Submenu: 'Selected as reference model...' sel
Tip: 'Export to BRF-readable (.smd) format, human skeleton.'
"""

__author__ = "Pelle Johnsen, edited by Eric Finley (Hellequin)"
__version__ = "Original 2.36 04/17/05, Edited 2.42 02/07/07"

__bpydoc__ = """\
This script exports Blender meshes to HalfLife2's .smd file format.

Supported:<br>

Missing:<br>
    Animation

Known issues:<br>
    Only interfaces with BRFEdit 0.8.2 and M&B 0.8xx human skeleton.
	Doesn't actually read the skeleton data, so if you've changed it, you'll still
	bind to the bones in their original locations.

Notes:<br>
    
"""

# $Id$
#
# --------------------------------------------------------------------------
# smd_export version 2.36.1
# Program versions: Blender 2.36 and HalfLife2
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2006: Pelle Johnsen, pelle.johnsen _at_ mail.dk
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

import Blender, meshtools, math, time
from Blender import Armature, Object, Scene, Mathutils, NMesh, Material, Image, Window, Modifier
from Blender.Mathutils import *

def writeTri(file, object, face, verts):
	file.write("Textures/%s\n" % face.image.name)
	for vi in verts:
		v = face.v[vi]
		vec = Vector([v.co[0], v.co[1], v.co[2], 1])
		vec = vec * object.getMatrix('worldspace')
		
		weights = object.getData().getVertexInfluences(v.index)
		maxWeight = 0.0
		parentBoneName = "0"
		if len(weights) > 0:
			for wi in weights:
				if wi[1] > maxWeight:
					maxWeight = wi[1]
					parentBoneName = wi[0]				

		file.write("%s %f %f %f " % (parentBoneName, vec.x, vec.y, vec.z))
		if face.smooth:
			vec = Vector([v.no[0], v.no[1], v.no[2], 1])
			vec = vec * object.getMatrix('worldspace')
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		else:
			vec = Vector([face.no[0], face.no[1], face.no[2], 1])
			vec = vec * object.getMatrix('worldspace')
			file.write("%f %f %f " % (vec.x, vec.y, vec.z))
		file.write("%f %f " % (face.uv[vi][0], face.uv[vi][1]))
		
		if len(weights) > 0:
			file.write("%d " % len(weights))
			for wi in weights:
				file.write("%s %f " % (wi[0], wi[1]))
		file.write("\n")

class SMDExport:

	def __init__(self, filename):
		print 'Trying SMDExport...'

		objects = Blender.Object.GetSelected()
	
		try:
			file = open(filename, 'w')
		except IOError, (errno, strerror):
			errmsg = "IOError #%s" % errno
			errmsg = errmsg + "%t|" + strerror
			Blender.Draw.PupMenu(errmsg)
			return None
		file.write("version 1\n")
		file.write("nodes\n")
		file.write('0 "0" -1\n')
		file.write('1 "1" 0\n')
		file.write('2 "2" 1\n')
		file.write('3 "3" 2\n')
		file.write('4 "4" 0\n')
		file.write('5 "5" 4\n')
		file.write('6 "6" 5\n')
		file.write('7 "7" 0\n')
		file.write('8 "8" 7\n')
		file.write('9 "9" 8\n')
		file.write('10 "10" 8\n')
		file.write('11 "11" 10\n')
		file.write('12 "12" 11\n')
		file.write('13 "13" 12\n')
		file.write('14 "14" 13\n')
		file.write('15 "15" 8\n')
		file.write('16 "16" 15\n')
		file.write('17 "17" 16\n')
		file.write('18 "18" 17\n')
		file.write('19 "19" 18\n')
		file.write('end\n')
		file.write('skeleton\n')
		file.write('time 0\n')
		file.write('0 -0.002980 -0.137334 8.603644 -1.298808 -1.320893 1.547939\n')
		file.write('1 0.704046 0.503604 0.999896 0.277594 0.049335 2.964254\n')
		file.write('2 4.651644 0.000002 0.000059 -0.032812 0.000261 -0.032812\n')
		file.write('3 4.724809 0.000001 -0.000000 1.537697 3.027819 1.537696\n')
		file.write('4 0.705008 -0.036968 -1.143362 0.279416 0.033027 3.011890\n')
		file.write('5 4.672235 -0.000000 -0.000052 -0.032637 0.004507 -0.032637\n')
		file.write('6 4.698491 -0.000000 -0.000001 1.487249 3.063254 1.487249\n')
		file.write('7 1.910835 -0.466635 0.143461 -0.241118 -0.067382 -0.240886\n')
		file.write('8 1.985241 0.000000 0.000026 0.000000 -0.000063 0.000000\n')
		file.write('9 2.499809 0.006891 -0.002300 0.000000 -0.015148 0.000000\n')
		file.write('10 1.758867 -0.001536 0.691073 0.000000 -1.540331 -0.000000\n')
		file.write('11 1.454170 0.000000 0.000001 0.000000 0.050534 0.000000\n')
		file.write('12 2.770649 0.000000 0.000001 -0.034646 -0.030769 -0.030201\n')
		file.write('13 2.719983 -0.022129 -0.000025 -0.254785 -0.181985 -0.254933\n')
		file.write('14 0.674834 -0.000000 0.000000 0.293284 0.085007 0.287551\n')
		file.write('15 1.758128 0.027817 -0.688622 0.000000 1.515255 0.000000\n')
		file.write('16 1.454170 -0.000000 0.000001 -0.000000 0.021417 0.000000\n')
		file.write('17 2.771103 0.007465 0.000086 0.019606 0.033890 -0.015856\n')
		file.write('18 2.717391 -0.018909 -0.000259 -0.257119 0.184048 -0.257016\n')
		file.write('19 0.681306 -0.000000 -0.000000 0.208913 -0.250179 0.205580\n')
		file.write('end\n')		
		file.write("triangles\n")
		
		for object in objects:
			if object.getType() != 'Mesh': continue
			print 'object:' + object.name
			mesh = object.getData()
			if not mesh.hasFaceUV():
				Blender.Draw.PupMenu("ERROR!%t|Meshses must have face UVs.|See Blender guide under UV editor.")
				return None
			faces = mesh.faces
			for face in faces:
				writeTri(file, object, face, [0, 1, 2])
				if len(face.v) == 4:
					writeTri(file, object, face, [0, 2, 3])

		file.write("end\n")
		file.close()

		print "Done. Saved to %s\n" % filename
			

# -- end of class SMDExport

def fs_callback(filename):
	if filename.find('.smd', -4) <= 0: filename += '.smd'
	test = SMDExport(filename)

if __script__['arg'] == 'config':
	exit
else:
	fname = Blender.sys.splitext(Blender.Get('filename'))[0] + '.smd'
	Blender.Window.FileSelector(fs_callback, "Export SMD", fname)